from Framework import Controller
from Framework import templatepage
from Framework.PluginManager import DeclarePlugin

from cherrypy import expose

from tableDataProvider import *
from guiProvider import *
from utils import *
from dbSchemaConsts import *

import globals
import sqlManager

from time import ctime

from os.path import join

RUNS_T = 'runsT'
REPACKER_T = 'repackerT'
REPACK_S_T = 'repackST'
MERGE_T = 'mergeT'
MERGE_S_T = 'mergeST'
RECO_T = 'recoT'
RECO_S_T = 'recoST'
RUN_REPACK_T = 'runRepackT'
RUN_RECO_T = 'runRecoT'
RUN_STREAMER_T = 'runStreamerT'
JOBS_T = 'jobsT'

TMPL_PATH = join(__file__.rsplit ("/", 1)[0], "Templates/")

class OldIndex (Controller):
    """
    Dummy page for old T0Mon address
    """
    #### Constructors ####

    def __init__(self, context):
        Controller.__init__(self, context, __file__)

    @expose
    def index(self, **kwds):
        return 'This page has moved <a href="/T0Mon/">here</a>'

class Index (Controller):
    #### Constructors ####

    def __init__(self, context):
        Controller.__init__(self, context, __file__)
        globals.logger = self.context.Logger()
        sqlManager.initBasicConnection()

    #### Public ####

    @expose

    def jobs(self, *args, **kwds):
        """
        Page displaying job lists
        """
        globals.args = args
        globals.kwds = kwds
        error = ""
        table = ""
        if globals.kwds.has_key('table'):
            table = str(kwds['table'])
        else: error += "<br />No table name specified"
        if globals.kwds.has_key('status'):
            try:
                status = int(kwds['status'])
            except ValueError:
                error += "<br />Status is not an int"
        else: error += "<br />No status flag specified"

        ordering = self._getTableOrdering(JOBS_T)
        if (ordering != None):
            orderby = ordering[0]
            asc = ordering[1]
        else :
            orderby = 0
            asc = False

        if table != "":
            if table == REPACK_JOBS_TABLE:
                jtd = RepackJobsDataProvider(orderby, asc, status = status)
            elif table == RECO_JOBS_TABLE:
                jtd = RecoJobsDataProvider(orderby, asc, status = status)
            elif table == MERGE_JOBS_TABLE:
                jtd = MergeJobsDataProvider(orderby, asc, status = status)
            else: error += "<br />Bad jobs table name specified"

        if error == "":
            jt = getTableGui(JOBS_T, jtd)
        else:
            jt = error

        return self.templatePage("jobs", {
            JOBS_T: jt,
            "time" : ctime(),
            "tmplPath" : TMPL_PATH
        })

    @expose
    def run(self, *args, **kwds):
        """
        Page displaying run statistics
        """
        globals.args = args
        globals.kwds = kwds

        if len(args) > 0:
            try:
                runid = int(args[0])
            except:
                return self.templatePage ("error", {
                    'msg': formatExceptionInfo(),
                    "tmplPath" : TMPL_PATH
                })
        else:
            return self.templatePage ("error", {
                'msg': "Run id not specified",
                "tmplPath" : TMPL_PATH
            })

        ordering = self._getTableOrdering(RUN_REPACK_T)
        if (ordering != None):
            orderby = ordering[0]
            asc = ordering[1]
        else :
            orderby = 0
            asc = False

        try:
            rrtd = RunRepackDataProvider(runid, orderby, asc)
            rrt = getTableGui(RUN_REPACK_T, rrtd)
        except: rrt = formatExceptionInfo()

        ordering = self._getTableOrdering(RUN_RECO_T)
        if (ordering != None):
            orderby = ordering[0]
            asc = ordering[1]
        else :
            orderby = 0
            asc = False

        try:
            rrtd = RunRecoDataProvider(runid, orderby, asc)
            rret = getTableGui(RUN_RECO_T, rrtd)
        except: rret = formatExceptionInfo()

        ordering = self._getTableOrdering(RUN_STREAMER_T)
        if (ordering != None):
            orderby = ordering[0]
            asc = ordering[1]
        else :
            orderby = 0
            asc = False

        try:
            rstd = RunStreamerDataProvider(runid, orderby, asc)
            rst = getTableGui(RUN_STREAMER_T, rstd)
        except: rst = formatExceptionInfo()

        #srgd = getStreamRepackGridData(runid)
        #srg = getGridGui(srgd)

        return self.templatePage("run", {
            RUN_REPACK_T : rrt,
            RUN_RECO_T : rret,
            RUN_STREAMER_T : rst,
            #"streamRepackG" : srg,
            "runid" : runid,
            "time" : ctime(),
            "tmplPath" : TMPL_PATH
        })

    @expose
    def index(self, **kwds):
        """
        Main T0Mon page
        """

        globals.kwds = kwds
        globals.args = None

        #### Runs table ####

        ordering = self._getTableOrdering(RUNS_T)
        if (ordering != None):
            orderby = ordering[0]
            asc = ordering[1]
        else :
            orderby = 0
            asc = False

        try:
            runsTableTd = RunsDataProvider(10, orderby, asc)
            runsTable = getTableGui(RUNS_T, runsTableTd)
        except: runsTable = formatExceptionInfo()

        #### Repacking status table ####

        ordering = self._getTableOrdering(REPACKER_T)
        if (ordering != None):
            orderby = ordering[0]
            asc = ordering[1]
        else :
            orderby = 0
            asc = False

        try:
            repackerTd = RepackJobsDataProvider(orderby, asc)
            repackerTable = getTableGui(REPACKER_T, repackerTd)
        except: repackerTable = formatExceptionInfo()

        #### Reconstruction status table ####

        ordering = self._getTableOrdering(RECO_T)
        if (ordering != None):
            orderby = ordering[0]
            asc = ordering[1]
        else :
            orderby = 0
            asc = False

        try:
            recoTd = RecoJobsDataProvider(orderby, asc)
            recoTable = getTableGui(RECO_T, recoTd)
        except: recoTable = formatExceptionInfo()

        #### Merging status table ####

        ordering = self._getTableOrdering(MERGE_T)
        if (ordering != None):
            orderby = ordering[0]
            asc = ordering[1]
        else :
            orderby = 0
            asc = False

        try:
            mergeTd = MergeJobsDataProvider(orderby, asc)
            mergeTable = getTableGui(MERGE_T, mergeTd)
        except: mergeTable = formatExceptionInfo()

        try:
            repackStatsTd = StatsDataProvider("Repack stats", REPACK_JOBS_TABLE)
            repackStatsTable = getVertTableGui(REPACK_S_T, repackStatsTd)
        except: repackStatsTable = formatExceptionInfo()

        try:
            recoStatsTd = StatsDataProvider("Reco stats", RECO_JOBS_TABLE)
            recoStatsTable = getVertTableGui(RECO_S_T, recoStatsTd)
        except: recoStatsTable = formatExceptionInfo()

        try:
            mergeStatsTd = StatsDataProvider("Merge stats", MERGE_JOBS_TABLE)
            mergeStatsTable = getVertTableGui(MERGE_S_T, mergeStatsTd)
        except: mergeStatsTable = formatExceptionInfo()

        return self.templatePage("index", {
            RUNS_T : runsTable,
            REPACKER_T : repackerTable,
            RECO_T : recoTable,
            MERGE_T : mergeTable,
            MERGE_S_T : mergeStatsTable,
            REPACK_S_T : repackStatsTable,
            RECO_S_T : recoStatsTable,
            "time" : ctime(),
            "tmplPath" : TMPL_PATH
        })

    #### Private ####

    def _getTableOrdering(self, tableid):
        """
        Method for parsing table ordering params from GET
        """
        if globals.kwds.has_key(tableid):
            asc = True
            orderby = 0
            words = globals.kwds[tableid].rsplit("_")
            try:
                orderby = int(words[0])
                if orderby < 0:
                    orderby = 0
                    self.context.Logger().error("Col id < 0")

            except ValueError:
                self.context.Logger().error("Could not convert table row id to an int")

            if (len(words) == 2):
                if (words[1] == "False"):
                    asc = False
            else:
                self.context.Logger().error("Badly formed table params")

            return [orderby, asc]
        else:
            return None

DeclarePlugin ("/Controllers/T0Mon/Index", Index, {"baseUrl": "/T0Mon"})
DeclarePlugin ("/Controllers/T0Mon/OldIndex", OldIndex, {"baseUrl": "/T0astmon"})
